package app.fedilab.openmaps.webview;

import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Build;
import android.provider.Settings;
import android.util.Base64;
import android.view.View;
import android.webkit.WebResourceError;
import android.webkit.WebResourceRequest;
import android.webkit.WebResourceResponse;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import androidx.annotation.RequiresApi;
import androidx.coordinatorlayout.widget.CoordinatorLayout;

import com.google.android.material.snackbar.Snackbar;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import app.fedilab.openmaps.MainActivity;
import app.fedilab.openmaps.R;
import app.fedilab.openmaps.helper.Helper;

public class OpenMapsWebViewClient extends WebViewClient {

    private Activity activity;
    private CoordinatorLayout rootView;

    public OpenMapsWebViewClient(Activity activity){
        this.activity = activity;
        rootView = activity.findViewById(R.id.main_layout);
    }

    @Override
    public boolean shouldOverrideUrlLoading(WebView view, String url) {
        // When user clicks a hyperlink, load in the existing WebView
        view.loadUrl(url);
        return true;
    }


    @Override
    public WebResourceResponse shouldInterceptRequest(final WebView view, String url) {

        if( url.contains("piwik.php") ) {
            ByteArrayInputStream nothing = new ByteArrayInputStream("".getBytes());
            return new WebResourceResponse("image/gif", "utf-8", nothing);
        }else if( Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP_MR1 && url.compareTo("https://www.mapcontrib.xyz/js/commons.bundle.js") == 0){
            InputStream commonJS = null;
            try {
                commonJS = activity.getAssets().open("commons.bundle.js");
            } catch (IOException e) {
                e.printStackTrace();
            }
            return new WebResourceResponse("text/javascript", "utf-8", commonJS);
        }else{
            return super.shouldInterceptRequest(view, url);
        }
    }

    private void injectScriptFile(WebView view, String scriptFile) {
        InputStream input;
        try {
            input = activity.getAssets().open(scriptFile);
            byte[] buffer = new byte[input.available()];
            //noinspection ResultOfMethodCallIgnored
            input.read(buffer);
            input.close();

            // String-ify the script byte-array using BASE64 encoding !!!
            String encoded = Base64.encodeToString(buffer, Base64.NO_WRAP);
            view.loadUrl("javascript:(function() {" +
                    "var parent = document.getElementsByTagName('head').item(0);" +
                    "var script = document.createElement('script');" +
                    "script.type = 'text/javascript';" +
                    // Tell the browser to BASE64-decode the string into your script !!!
                    "script.innerHTML = window.atob('" + encoded + "');" +
                    "parent.appendChild(script)" +
                    "})()");
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }


    private void injectCSS(WebView view, String cssFile) {
        try {
            InputStream inputStream = activity.getAssets().open(cssFile);
            byte[] buffer = new byte[inputStream.available()];
            inputStream.read(buffer);
            inputStream.close();
            String encoded = Base64.encodeToString(buffer, Base64.NO_WRAP);
            view.loadUrl("javascript:(function() {" +
                    "var parent = document.getElementsByTagName('head').item(0);" +
                    "var style = document.createElement('style');" +
                    "style.type = 'text/css';" +
                    // Tell the browser to BASE64-decode the string into your script !!!
                    "style.innerHTML = window.atob('" + encoded + "');" +
                    "parent.appendChild(style)" +
                    "})()");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onPageStarted(WebView view, String url, Bitmap favicon) {
        super.onPageStarted(view, url, favicon);

        if( url.contains(Helper.hydrant_contrib_map)){
            injectScriptFile(view, "js/optin.js");
        }
        ((MainActivity)activity).showProgressDialog();
    }


    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    public void onReceivedError(WebView view, WebResourceRequest request,
                                WebResourceError error) {


        if (isConnected()) {
            if( error.getDescription() != null && error.getDescription().length() > 0 && !error.getDescription().toString().trim().equals("Not Found") && !error.getDescription().toString().trim().equals("Internal Server Error")) {
                final Snackbar snackBar = Snackbar.make(rootView, activity.getString(R.string.error, error.getDescription()), Snackbar.LENGTH_INDEFINITE);
                snackBar.setAction(activity.getString(R.string.reload), new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        view.loadUrl("javascript:window.location.reload( true )");
                    }
                });
                snackBar.show();
            }
        } else {
            final Snackbar snackBar = Snackbar.make(rootView, activity.getString(R.string.no_internet), Snackbar.LENGTH_INDEFINITE);
            snackBar.setAction(activity.getString(R.string.enable_data), new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    activity.startActivityForResult(new Intent(Settings.ACTION_WIRELESS_SETTINGS), 0);
                    view.loadUrl("javascript:window.location.reload( true )");
                    snackBar.dismiss();
                }
            });
            snackBar.show();
        }

        super.onReceivedError(view, request, error);

    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    @Override
    public void onReceivedHttpError(WebView view,
                                    WebResourceRequest request, WebResourceResponse errorResponse) {

        if (isConnected()) {
            if( errorResponse.getReasonPhrase() != null && errorResponse.getReasonPhrase().length() > 0 && !errorResponse.getReasonPhrase().trim().equals("Not Found") && !errorResponse.getReasonPhrase().trim().equals("Internal Server Error")) {
                final Snackbar snackBar = Snackbar.make(rootView, activity.getString(R.string.error,  errorResponse.getReasonPhrase()) , Snackbar.LENGTH_INDEFINITE);

                snackBar.setAction(activity.getString(R.string.reload), new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        view.loadUrl("javascript:window.location.reload( true )");
                    }
                });
                snackBar.show();
            }
        } else {
            final Snackbar snackBar = Snackbar.make(rootView, activity.getString(R.string.no_internet), Snackbar.LENGTH_INDEFINITE);
            snackBar.setAction(activity.getString(R.string.enable_data), new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    activity.startActivityForResult(new Intent(Settings.ACTION_WIRELESS_SETTINGS), 0);
                    view.loadUrl("javascript:window.location.reload( true )");
                    snackBar.dismiss();
                }
            });
            snackBar.show();
        }
        super.onReceivedHttpError(view, request, errorResponse);
    }


    /**
     * Check if there is any connectivity
     *
     * @return is Device Connected
     */
    private boolean isConnected() {

        ConnectivityManager cm = (ConnectivityManager)
                activity.getSystemService(Context.CONNECTIVITY_SERVICE);

        if (null != cm) {
            NetworkInfo info = cm.getActiveNetworkInfo();
            return (info != null && info.isConnected());
        }

        return false;

    }

    @Override
    public void onPageFinished(WebView view, String url) {
        //injectScriptFile(view, "test.js");
        injectCSS(view, "css/custom.css");

        ((MainActivity)activity).hideProgressDialog();
    }
}
